###############################################################################
# Tests for chemkin.thermodynamics.thermo module
###############################################################################

from chemkin import pckg_xml_path
from chemkin.chemkin_errors import ChemKinError
from chemkin.preprocessing.parse_xml import XmlParser

def test_get_backward_coefs_normal():
	Ti = [750, 1500]
	xml_parser = XmlParser(pckg_xml_path('rxns_reversible'))
	parsed_data_list = xml_parser.parsed_data_list(Ti) # calling Thermo().get_backward_coefs()
	assert parsed_data_list[0]['T'] == 750
	assert parsed_data_list[1]['T'] == 1500

def test_get_backward_coefs_high_range_err():
	Ti = [5000]
	xml_parser = XmlParser(pckg_xml_path('rxns_reversible'))
	parsed_data_list = xml_parser.parsed_data_list(Ti) # calling Thermo().get_backward_coefs()
	assert parsed_data_list[0]['b_ki'] == 'Not Defined'

def test_get_backward_coefs_low_range_err():
	Ti = [100]
	xml_parser = XmlParser(pckg_xml_path('rxns_reversible'))
	parsed_data_list = xml_parser.parsed_data_list(Ti) # calling Thermo().get_backward_coefs()
	assert parsed_data_list[0]['b_ki'] == 'Not Defined'


from chemkin.reaction.elementary_rxn import IrreversibleElementaryRxn
from chemkin.reaction.elementary_rxn import ReversibleElementaryRxn

def print_reaction_rate(parsed_data_list, xi):
	test_flag = 0 # reation rates can be printed
	
	for parsed_data in parsed_data_list:

		species = parsed_data['species']
		ki = parsed_data['ki']
		sys_vi_p = parsed_data['sys_vi_p']
		sys_vi_dp = parsed_data['sys_vi_dp']
		is_reversible = parsed_data['is_reversible']
		T = parsed_data['T']
	
		if is_reversible == False:
			rxn_rates = IrreversibleElementaryRxn(ki, xi, sys_vi_p, sys_vi_dp).reaction_rate()
		
		else:
			b_ki = parsed_data['b_ki']
			if str(b_ki) == 'Not Defined':
				test_flag = 1 # reaction rates cannot be printed because T is not in some specie's temperature range 
				
				print('------At Temperature', T, 'K------')
				print('Backward reaction coefficients not defined: T={} is not in some specie\'s temperature range.'.format(T))
				print('--------------------------------')
				continue
			
			rxn_rates = ReversibleElementaryRxn(ki, b_ki, xi, sys_vi_p, sys_vi_dp).reaction_rate()

		
		print('------At Temperature', T, 'K------')
		for s, rate in zip(species, rxn_rates):
			print('    ', s, ':', rate)
		print('--------------------------------')

	return test_flag


import os.path
import sqlite3
import numpy as np
from chemkin.chemkin_errors import ChemKinError

class Thermo():
    """ Class of thermodynanmics
    """

    def __init__ (self, species, T, ki, vi_p, vi_dp, db_name='NASA_coef.sqlite'):
        self.p0 = 10e5
        self.R = 8.314
        self.species = species
        self.T = T
        self.ki = ki
        self.b_ki = None
        self.vi_p = vi_p
        self.vi_dp = vi_dp
        self.vi = np.matrix(self.vi_dp).T - np.matrix(self.vi_p).T  # calculate overall stoicheometric coefficients
        self.gamma = np.sum(self.vi, axis=0)
        self.dao = ThermoDAO(db_name)


    def get_backward_coefs (self):
        species_high = self.dao.get_species(self.T, 'high')
        species_low = self.dao.get_species(self.T, 'low')

        H_over_RT_species = []
        S_over_R_species = []
        for s in self.species:
            if self.T >= 1000: # high temperature range
                if s not in species_high:
                    raise ChemKinError('Thermo.get_backward_coefs()',
                        'The specie {}\'s high temperature bound is not higher than the current T={}.'.format(s, self.T))
                NASA_coefs = self.dao.get_coeffs(s, 'high')
            else: # low temperature range
                if s not in species_low:
                    raise ChemKinError('Thermo.get_backward_coefs()',
                        'The specie {}\'s low temperature bound is not lower than the current T={}.'.format(s, self.T))
                NASA_coefs = self.dao.get_coeffs(s, 'low')

            # Compute enthalpy/(RT) of a specie
            H_T_arr = np.array([1, 1/2*self.T, 1/3*(self.T**2), 1/4*(self.T**3), 1/5*(self.T**4), 1/self.T])
            H_coef_arr = np.array([NASA_coefs[0], NASA_coefs[1], NASA_coefs[2], NASA_coefs[3], NASA_coefs[4], NASA_coefs[5]])
            H_over_RT_species.append(np.dot(H_T_arr, H_coef_arr))

            # Compute entropy/R of a specie
            S_T_arr = np.array([np.log(self.T), self.T, 1/2*(self.T**2), 1/3*(self.T**3), 1/4*(self.T**4), 1])
            S_coef_arr = np.array([NASA_coefs[0], NASA_coefs[1], NASA_coefs[2], NASA_coefs[3], NASA_coefs[4], NASA_coefs[6]])
            S_over_R_species.append(np.dot(S_T_arr, S_coef_arr))

        delta_H_over_RT = np.squeeze(np.asarray(np.dot(H_over_RT_species, self.vi))) # delta enthalpy of a system of reactions
        delta_S_over_R = np.squeeze(np.asarray(np.dot(S_over_R_species, self.vi))) # delta entropy of a system of reactions
        
        ke = np.squeeze(np.asarray(np.multiply(np.power(self.p0/(self.R*self.T), self.gamma), np.exp(delta_S_over_R-delta_H_over_RT))))

        self.b_ki = self.ki/ke

        return self.b_ki

class ThermoDAO():
    """ Database Access Object for thermodynanmics
    """

    def __init__ (self, db_name):
        BASE_DIR = os.path.dirname(os.path.abspath(__file__))
        self.db_path = os.path.join(BASE_DIR, db_name)

    def get_coeffs (self, species_name, temp_range):
        db = sqlite3.connect(self.db_path)
        cursor = db.cursor()
        query = '''SELECT COEFF_1, COEFF_2, COEFF_3, COEFF_4, COEFF_5, COEFF_6, COEFF_7
                    FROM {} 
                    WHERE SPECIES_NAME = "{}"'''.format(temp_range.upper(), species_name)
        coeffs = list(cursor.execute(query).fetchall()[0])
        db.commit()
        db.close()
        return coeffs

    def get_species(self, temp, temp_range):
        db = sqlite3.connect(self.db_path)
        cursor = db.cursor()
        if temp_range == 'low': # temp_range == 'low'
            query = '''SELECT SPECIES_NAME FROM {} WHERE TLOW < {}'''.format(temp_range.upper(), temp)
        else: # temp_range == 'high'
            query = '''SELECT SPECIES_NAME FROM {} WHERE THIGH > {}'''.format(temp_range.upper(), temp)
        species = []
        for s in cursor.execute(query).fetchall():
            species.append(s[0])
        db.commit()
        db.close()
        return species

if __name__ == "__main__":
    import doctest
    doctest.testmod(verbose=True)